// ==========================================================================
// NumLock.cpp: implementation of the CNumLock class, for details see:
//              NumLock.h
//
// Copyright:   1999 - Wilfried Roemer, Berlin, all rights reserved
// ==========================================================================

#include "stdafx.h"
#include "NumLock.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// ==========================================================================
// Construction/Destruction
// ==========================================================================
CNumLock::CNumLock( bool bAlwaysOn ) :
  m_bIgnoreKeyInput( false )
  {
  // initialize required state

  if( bAlwaysOn )
    m_iReqdNumLockState = 0x0001;
  else
    m_iReqdNumLockState = 0x0000;

  // initialize static info structure

  m_OSVersionInfo.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
  VERIFY( GetVersionEx( &m_OSVersionInfo ));
  }

CNumLock::~CNumLock()
  {
  }
// ==========================================================================
// Toggle NumLock - reverses the current NumLock status. The handling
// is different for Windows 9x and Windows NT.
// ==========================================================================
void CNumLock::ToggleNumLock()
  {
  _ASSERT( m_bIgnoreKeyInput == false );
  m_bIgnoreKeyInput = true;

  if( m_OSVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
    {
    BYTE KeyboardState[ 256 ];
    VERIFY( GetKeyboardState( KeyboardState ));
    KeyboardState[ VK_NUMLOCK ] = KeyboardState[ VK_NUMLOCK ] ^ 1;
    VERIFY( SetKeyboardState( KeyboardState ));
    }
  else if( m_OSVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT )
    { 
    keybd_event( VK_NUMLOCK, 0x45, KEYEVENTF_EXTENDEDKEY | 0, 0 );
    keybd_event( VK_NUMLOCK, 0x45, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0 );
    }
  else
    ; // OS not supported

  m_bIgnoreKeyInput = false;
  }
// ==========================================================================
// Check NumLock State - to be used in the main applications OnIdle handler:
// this checks whether the NumLock state is still as requested
// ==========================================================================
void CNumLock::CheckNumLockState()
  {
  // if this is called while ignoring any keyboard input, the current state
  // of the numlock key is already being changed, hence:

  if( m_bIgnoreKeyInput )
    return;

  if( GetCurrNumLockState() == m_iReqdNumLockState )
    ; // nothing to do
  else
    ToggleNumLock();
  }
// ==========================================================================
// Filter Message - filter VK_NUMLOCK messages: if NumLock is down, ignore all
// further WM_KEYDOWN messages until VK_NUMLOCK is up again. Return true if
// message was complete handled (processed or to be dropped), false if
// message should be processed further.
// ==========================================================================
bool CNumLock::FilterMessage( const MSG* pMsg )
  {
  if( pMsg->message == WM_KEYDOWN )
    {
    if( m_bIgnoreKeyInput )
      return true;

    else if( pMsg->wParam == VK_NUMLOCK )
      return m_bIgnoreKeyInput = true;
    else
      return false;
    }
  else if( pMsg->message == WM_KEYUP )
    {
    if( pMsg->wParam == VK_NUMLOCK )
      {
      m_bIgnoreKeyInput = false;
      CheckNumLockState();
      return true;
      }
    else
      return m_bIgnoreKeyInput;
    }
  else
    return false;
  }
